@@ -9,6 +9,7 @@ from click import UsageError
 from datetime import timedelta
 
 import mlflow.db
+from mlflow.entities import ViewType
 import mlflow.experiments
 import mlflow.deployments.cli
 from mlflow import projects
@@ -134,6 +135,16 @@ def cli():
     help="The name to give the MLflow Run associated with the project execution. If not specified, "
     "the MLflow Run name is left unset.",
 )
+@click.option(
+    "--skip-image-build",
+    is_flag=True,
+    default=False,
+    show_default=True,
+    help=(
+        "Only valid for Docker projects. If specified, skips building a new Docker image and "
+        "directly uses the image specified by the `image` field in the MLproject file."
+    ),
+)
 def run(
     uri,
     entry_point,
@@ -149,6 +160,7 @@ def run(
     storage_dir,
     run_id,
     run_name,
+    skip_image_build,
 ):
     """
     Run an MLflow project from the given URI.
@@ -195,6 +207,7 @@ def run(
             synchronous=backend in ("local", "kubernetes") or backend is None,
             run_id=run_id,
             run_name=run_name,
+            skip_image_build=skip_image_build,
         )
     except projects.ExecutionException as e:
         _logger.error("=== %s ===", e)
@@ -519,17 +532,37 @@ def server(
     " are not specified, data is removed for all runs in the `deleted`"
     " lifecycle stage.",
 )
-def gc(older_than, backend_store_uri, run_ids):
+@click.option(
+    "--experiment-ids",
+    default=None,
+    help="Optional comma separated list of experiments to be permanently deleted including "
+    "all of their associated runs. If experiment ids are not specified, data is removed for all "
+    "experiments in the `deleted` lifecycle stage.",
+)
+def gc(older_than, backend_store_uri, run_ids, experiment_ids):
     """
     Permanently delete runs in the `deleted` lifecycle stage from the specified backend store.
     This command deletes all artifacts and metadata associated with the specified runs.
     """
+    import warnings
+    from mlflow.utils.time_utils import get_current_time_millis
+
     backend_store = _get_store(backend_store_uri, None)
+    skip_experiments = False
     if not hasattr(backend_store, "_hard_delete_run"):
         raise MlflowException(
             "This cli can only be used with a backend that allows hard-deleting runs"
         )
 
+    if not hasattr(backend_store, "_hard_delete_experiment"):
+        warnings.warn(
+            "The specified backend does not allow hard-deleting experiments. Experiments"
+            " will be skipped.",
+            FutureWarning,
+            stacklevel=2,
+        )
+        skip_experiments = True
+
     time_delta = 0
 
     if older_than is not None:
@@ -550,16 +583,53 @@ def gc(older_than, backend_store_uri, run_ids):
     deleted_run_ids_older_than = backend_store._get_deleted_runs(older_than=time_delta)
     if not run_ids:
         run_ids = deleted_run_ids_older_than
-
     else:
         run_ids = run_ids.split(",")
 
-    for run_id in run_ids:
+    if not skip_experiments:
+        deleted_older_experiment_ids = []
+        next_experiment_page_token = None
+        time_threshold = get_current_time_millis() - time_delta
+        filter_string = f"last_update_time < {time_threshold}" if older_than else ""
+        while True:
+            page_results = backend_store.search_experiments(
+                view_type=ViewType.DELETED_ONLY,
+                filter_string=filter_string,
+                page_token=next_experiment_page_token,
+            )
+            for experiment in page_results:
+                deleted_older_experiment_ids.append(experiment.experiment_id)
+            if page_results.token is None:
+                break
+            next_experiment_page_token = page_results.token
+
+        if experiment_ids:
+            experiment_ids = experiment_ids.split(",")
+        else:
+            experiment_ids = deleted_older_experiment_ids
+
+        next_run_page_token = None
+        while True:
+            page_results = backend_store.search_runs(
+                experiment_ids=experiment_ids,
+                filter_string="",
+                run_view_type=ViewType.DELETED_ONLY,
+                page_token=next_run_page_token,
+            )
+
+            for run in page_results:
+                run_ids.append(run.info.run_id)
+
+            if page_results.token is None:
+                break
+            next_run_page_token = page_results.token
+
+    for run_id in set(run_ids):
         run = backend_store.get_run(run_id)
         if run.info.lifecycle_stage != LifecycleStage.DELETED:
             raise MlflowException(
-                "Run % is not in `deleted` lifecycle stage. Only runs in "
-                "`deleted` lifecycle stage can be deleted." % run_id
+                "Run % is not in `deleted` lifecycle stage. Only runs in"
+                " `deleted` lifecycle stage can be deleted." % run_id
             )
         # raise MlflowException if run_id is newer than older_than parameter
         if older_than and run_id not in deleted_run_ids_older_than:
@@ -573,6 +643,23 @@ def gc(older_than, backend_store_uri, run_ids):
         backend_store._hard_delete_run(run_id)
         click.echo("Run with ID %s has been permanently deleted." % str(run_id))
 
+    if not skip_experiments:
+        invalid_experiments = []
+        for experiment_id in experiment_ids:
+            if older_than and experiment_id not in deleted_older_experiment_ids:
+                invalid_experiments.append(experiment_id)
+
+        if invalid_experiments:
+            raise MlflowException(
+                f"Experiments {invalid_experiments} are not older than the required age. "
+                f"Only runs older than {older_than} can be deleted.",
+                error_code=INVALID_PARAMETER_VALUE,
+            )
+
+        for experiment_id in experiment_ids:
+            backend_store._hard_delete_experiment(experiment_id)
+            click.echo("Experiment with ID %s has been permanently deleted." % str(experiment_id))
+
 
 cli.add_command(mlflow.deployments.cli.commands)
 cli.add_command(mlflow.experiments.commands)
