@@ -85,6 +85,11 @@ MLFLOW_HTTP_REQUEST_BACKOFF_FACTOR = _EnvironmentVariable(
 #: (default: ``120``)
 MLFLOW_HTTP_REQUEST_TIMEOUT = _EnvironmentVariable("MLFLOW_HTTP_REQUEST_TIMEOUT", int, 120)
 
+#: Specifies whether MLFlow HTTP requests should be signed using AWS signature V4. It will overwrite
+#: (default: ``False``). When set, it will overwrite the "Authorization" HTTP header
+# See https://docs.aws.amazon.com/general/latest/gr/signature-version-4.html for more information.
+MLFLOW_TRACKING_AWS_SIGV4 = _BooleanEnvironmentVariable("MLFLOW_TRACKING_AWS_SIGV4", False)
+
 #: Specifies the chunk size to use when downloading a file from GCS
 #: (default: ``None``). If None, the chunk size is automatically determined by the
 #: ``google-cloud-storage`` package.
@@ -156,3 +161,17 @@ MLFLOW_SQLALCHEMYSTORE_MAX_OVERFLOW = _EnvironmentVariable(
 #: for more information.
 #: (default: ``False``)
 MLFLOW_SQLALCHEMYSTORE_ECHO = _BooleanEnvironmentVariable("MLFLOW_SQLALCHEMYSTORE_ECHO", False)
+
+#: Specifies the ``poolclass`` parameter to use for ``sqlalchemy.create_engine`` in the
+#: SQLAlchemy tracking store. See https://docs.sqlalchemy.org/en/14/core/engines.html#sqlalchemy.create_engine.params.poolclass
+#: for more information.
+#: (default: ``None``)
+MLFLOW_SQLALCHEMYSTORE_POOLCLASS = _EnvironmentVariable(
+    "MLFLOW_SQLALCHEMYSTORE_POOLCLASS", str, None
+)
+
+#: Specifies the ``timeout_seconds`` for MLflow Model dependency inference operations.
+#: (default: ``120``)
+MLFLOW_REQUIREMENTS_INFERENCE_TIMEOUT = _EnvironmentVariable(
+    "MLFLOW_REQUIREMENTS_INFERENCE_TIMEOUT", int, 120
+)
