@@ -43,6 +43,8 @@ from mlflow.utils.environment import (
     _CONDA_ENV_FILE_NAME,
     _REQUIREMENTS_FILE_NAME,
     _CONSTRAINTS_FILE_NAME,
+    _PYTHON_ENV_FILE_NAME,
+    _PythonEnv,
 )
 from mlflow.utils.requirements_utils import _get_pinned_requirement
 from mlflow.utils.file_utils import write_to
@@ -208,6 +210,8 @@ def save_model(
     # Save `requirements.txt`
     write_to(os.path.join(path, _REQUIREMENTS_FILE_NAME), "\n".join(pip_requirements))
 
+    _PythonEnv.current().to_yaml(os.path.join(path, _PYTHON_ENV_FILE_NAME))
+
 
 def _save_model(lgb_model, model_path):
     """
@@ -368,11 +372,28 @@ class _LGBModelWrapper:
         return self.lgb_model.predict(dataframe)
 
 
+def _patch_metric_names(metric_dict):
+    # lightgbm provides some metrics with "@", e.g. "ndcg@3" that are not valid MLflow metric names
+    patched_metrics = {
+        metric_name.replace("@", "_at_"): value for metric_name, value in metric_dict.items()
+    }
+    changed_keys = set(patched_metrics.keys()) - set(metric_dict.keys())
+    if changed_keys:
+        _logger.info(
+            "Identified one or more metrics with names containing the invalid character `@`."
+            " These metric names have been sanitized by replacing `@` with `_at_`, as follows: %s",
+            ", ".join(changed_keys),
+        )
+
+    return patched_metrics
+
+
 def _autolog_callback(env, metrics_logger, eval_results):
     res = {}
     for data_name, eval_name, value, _ in env.evaluation_result_list:
         key = data_name + "-" + eval_name
         res[key] = value
+    res = _patch_metric_names(res)
     metrics_logger.record_metrics(res, env.iteration)
     eval_results.append(res)
 
@@ -593,7 +614,7 @@ def autolog(
                     "will ignore the failure and continue. Exception: "
                 )
 
-            imp = {ft: imp for ft, imp in zip(features, importance.tolist())}
+            imp = dict(zip(features, importance.tolist()))
             tmpdir = tempfile.mkdtemp()
             try:
                 filepath = os.path.join(tmpdir, "feature_importance_{}.json".format(imp_type))
