@@ -38,41 +38,43 @@ RUN python /tmp/get-pip.py
 RUN pip install virtualenv
 """
 
-if os.getenv("http_proxy") is not None and os.getenv("https_proxy") is not None:
+
+def _get_maven_proxy():
+    http_proxy = os.getenv("http_proxy")
+    https_proxy = os.getenv("https_proxy")
+    if not http_proxy or not https_proxy:
+        return ""
 
     # Expects proxies as either PROTOCOL://{USER}:{PASSWORD}@HOSTNAME:PORT
     # or PROTOCOL://HOSTNAME:PORT
-    parsed_http_proxy = urlparse(os.environ["http_proxy"])
+    parsed_http_proxy = urlparse(http_proxy)
     assert parsed_http_proxy.hostname is not None, "Invalid `http_proxy` hostname."
-    assert isinstance(parsed_http_proxy.port, int), f"Invalid Proxy Port: {parsed_http_proxy.port}"
+    assert parsed_http_proxy.port is not None, f"Invalid proxy port: {parsed_http_proxy.port}"
 
-    parsed_https_proxy = urlparse(os.environ["https_proxy"])
+    parsed_https_proxy = urlparse(https_proxy)
     assert parsed_https_proxy.hostname is not None, "Invalid `https_proxy` hostname."
-    assert isinstance(
-        parsed_https_proxy.port, int
-    ), f"Invalid Proxy Port: {parsed_https_proxy.port}"
+    assert parsed_https_proxy.port is not None, f"Invalid proxy port: {parsed_https_proxy.port}"
 
-    MAVEN_PROXY = (
-        " -DproxySet=true -Dhttp.proxyHost={http_proxy_host} "
-        "-Dhttp.proxyPort={http_proxy_port} -Dhttps.proxyHost={https_proxy_host} "
-        "-Dhttps.proxyPort={https_proxy_port} -Dhttps.nonProxyHosts=repo.maven.apache.org"
-    ).format(
-        http_proxy_host=parsed_http_proxy.hostname,
-        http_proxy_port=parsed_http_proxy.port,
-        https_proxy_host=parsed_https_proxy.hostname,
-        https_proxy_port=parsed_https_proxy.port,
+    maven_proxy_options = (
+        "-DproxySet=true",
+        f"-Dhttp.proxyHost={parsed_http_proxy.hostname}",
+        f"-Dhttp.proxyPort={parsed_http_proxy.port}",
+        f"-Dhttps.proxyHost={parsed_https_proxy.hostname}",
+        f"-Dhttps.proxyPort={parsed_https_proxy.port}",
+        "-Dhttps.nonProxyHosts=repo.maven.apache.org",
     )
 
-    if parsed_http_proxy.username is not None and parsed_http_proxy.password is not None:
+    if parsed_http_proxy.username is None or parsed_http_proxy.password is None:
+        return " ".join(maven_proxy_options)
 
-        MAVEN_PROXY += (
-            " -Dhttp.proxyUser={proxy_username} -Dhttp.proxyPassword={proxy_password}".format(
-                proxy_username=parsed_http_proxy.username, proxy_password=parsed_http_proxy.password
-            )
+    return " ".join(
+        (
+            *maven_proxy_options,
+            f"-Dhttp.proxyUser={parsed_http_proxy.username}",
+            f"-Dhttp.proxyPassword={parsed_http_proxy.password}",
         )
+    )
 
-else:
-    MAVEN_PROXY = ""  # No Proxy
 
 DISABLE_ENV_CREATION = "MLFLOW_DISABLE_ENV_CREATION"
 
@@ -120,6 +122,7 @@ def _get_mlflow_install_step(dockerfile_context_dir, mlflow_home):
     Get docker build commands for installing MLflow given a Docker context dir and optional source
     directory
     """
+    maven_proxy = _get_maven_proxy()
     if mlflow_home:
         mlflow_dir = _copy_project(src_path=mlflow_home, dst_path=dockerfile_context_dir)
         return (
@@ -130,7 +133,7 @@ def _get_mlflow_install_step(dockerfile_context_dir, mlflow_home):
             "mkdir -p /opt/java/jars && "
             "mv /opt/mlflow/mlflow/java/scoring/target/"
             "mlflow-scoring-*-with-dependencies.jar /opt/java/jars\n"
-        ).format(mlflow_dir=mlflow_dir, maven_proxy=MAVEN_PROXY)
+        ).format(mlflow_dir=mlflow_dir, maven_proxy=maven_proxy)
     else:
         return (
             "RUN pip install mlflow=={version}\n"
@@ -146,7 +149,38 @@ def _get_mlflow_install_step(dockerfile_context_dir, mlflow_home):
             "RUN cd /opt/java && mvn "
             "--batch-mode dependency:copy-dependencies "
             "-DoutputDirectory=/opt/java/jars {maven_proxy}\n"
-        ).format(version=mlflow.version.VERSION, maven_proxy=MAVEN_PROXY)
+        ).format(version=mlflow.version.VERSION, maven_proxy=maven_proxy)
+
+
+def _generate_dockerfile_content(
+    setup_miniconda, setup_pyenv_and_virtualenv, install_mlflow, custom_setup_steps, entrypoint
+):
+    """
+    Generates a Dockerfile that can be used to build a docker image, that serves ML model
+    stored and tracked in MLflow.
+
+    It just takes string parameters containing docker imperatives and has no logic
+    whatsoever. It will be more convenient if a more sophisticated function
+    with some boolean flags would be called `generate_dockerfile`
+    while this function being a backend of sorts for such function.
+
+    :param setup_miniconda: Docker instructions related to set up miniconda. If used at all,
+    variable `SETUP_MINICONDA` provides a working template for instructions. Should be either an
+    empty string or `SETUP_MINICONDA`-based instructions :param setup_pyenv_and_virtualenv:
+    Docker instructions related to set up pyenv and virtualenv. If used at all, variable
+    `SETUP_PYENV_AND_VIRTUALENV` provides a working template for instructions. Should be either
+    an empty string or `SETUP_PYENV_AND_VIRTUALENV`-based :param install_mlflow: Docker
+    instruction for installing MLflow in given Docker context dir and optional source directory
+    :param custom_setup_steps: Docker instructions for any customizations in the resulting
+    Dockerfile :param entrypoint: String containing ENTRYPOINT directive for docker image
+    """
+    return _DOCKERFILE_TEMPLATE.format(
+        setup_miniconda=setup_miniconda,
+        setup_pyenv_and_virtualenv=setup_pyenv_and_virtualenv,
+        install_mlflow=install_mlflow,
+        custom_setup_steps=custom_setup_steps,
+        entrypoint=entrypoint,
+    )
 
 
 def _build_image(
@@ -178,7 +212,7 @@ def _build_image(
         custom_setup_steps = custom_setup_steps_hook(cwd) if custom_setup_steps_hook else ""
         with open(os.path.join(cwd, "Dockerfile"), "w") as f:
             f.write(
-                _DOCKERFILE_TEMPLATE.format(
+                _generate_dockerfile_content(
                     setup_miniconda=setup_miniconda,
                     setup_pyenv_and_virtualenv=setup_pyenv_and_virtualenv,
                     install_mlflow=install_mlflow,
@@ -187,7 +221,6 @@ def _build_image(
                 )
             )
         _logger.info("Building docker image with name %s", image_name)
-        os.system("find {cwd}/".format(cwd=cwd))
         _build_image_from_context(context_dir=cwd, image_name=image_name)
 
 
