@@ -70,6 +70,16 @@ class FlavorBackend:
         """
         pass
 
+    @abstractmethod
+    def build_image(self, model_uri, image_name, install_mlflow, mlflow_home, enable_mlserver):
+        raise NotImplementedError
+
+    @abstractmethod
+    def generate_dockerfile(
+        self, model_uri, output_path, install_mlflow, mlflow_home, enable_mlserver
+    ):
+        raise NotImplementedError
+
     @abstractmethod
     def can_score_model(self):
         """
