@@ -31,6 +31,8 @@ from mlflow.utils.environment import (
     _CONDA_ENV_FILE_NAME,
     _REQUIREMENTS_FILE_NAME,
     _CONSTRAINTS_FILE_NAME,
+    _PYTHON_ENV_FILE_NAME,
+    _PythonEnv,
 )
 from mlflow.utils.requirements_utils import _get_pinned_requirement
 from mlflow.utils.file_utils import write_to
@@ -199,6 +201,8 @@ def save_model(
     # Save `requirements.txt`
     write_to(os.path.join(path, _REQUIREMENTS_FILE_NAME), "\n".join(pip_requirements))
 
+    _PythonEnv.current().to_yaml(os.path.join(path, _PYTHON_ENV_FILE_NAME))
+
 
 def _load_model(model_file):
     import onnx
@@ -226,14 +230,14 @@ class _OnnxModelWrapper:
             providers = ONNX_EXECUTION_PROVIDERS
 
         # NOTE: Some distributions of onnxruntime require the specification of the providers
-        # argument on calling. E.g. onnxruntime-gpu. The package import call does not differnetiate
+        # argument on calling. E.g. onnxruntime-gpu. The package import call does not differentiate
         #  which architecture specific version has been installed, as all are imported with
         # onnxruntime. onnxruntime documentation says that from v1.9.0 some distributions require
         #  the providers list to be provided on calling an InferenceSession. Therefore the try
         #  catch structure below attempts to create an inference session with just the model path
         #  as pre v1.9.0. If that fails, it will use the providers list call.
         # At the moment this is just CUDA and CPU, and probably should be expanded.
-        # A method of user customisation has been provided by adding a variable in the save_model()
+        # A method of user customization has been provided by adding a variable in the save_model()
         # function, which allows the ability to pass the list of execution providers via a
         # optional argument e.g.
         #
