@@ -14,7 +14,7 @@ The training pipeline contains the following sequential steps:
 
 The batch scoring pipeline contains the following sequential steps:
 
-**ingest** -> **predict**
+**ingest_scoring** -> **predict**
 
 The pipeline steps are defined as follows:
 
@@ -61,6 +61,11 @@ The pipeline steps are defined as follows:
         performance metrics, and lineage information are logged to MLflow Tracking, producing
         an MLflow Run.
 
+            .. note::
+                The **train** step supports hyperparameter tuning with hyperopt by adding 
+                configurations in the 
+                |'tuning' section of the 'train' step definition in pipeline.yaml|. 
+
    - **evaluate**
       - The **evaluate** step evaluates the model pipeline created by the **train** step on
         the test dataset output from the **split** step, computing performance metrics and
@@ -91,14 +96,32 @@ The pipeline steps are defined as follows:
                 always registered with the MLflow Model Registry when the **register** step is
                 executed.
 
-    - **predict**
-      - The **predict** step
+   - **ingest_scoring**
+      - The **ingest_scoring** step resolves the dataset specified by the 
+        |'data/scoring' section in pipeline.yaml| and converts it to parquet format, leveraging 
+        the custom dataset parsing code defined in |steps/ingest.py| if necessary. 
+    
+            .. note::
+                If you make changes to the dataset referenced by the **ingest_scoring** step 
+                (e.g. by adding new records or columns), you must manually re-run the 
+                **ingest_scoring** step in order to use the updated dataset in the pipeline. 
+                The **ingest_scoring** step does *not* automatically detect changes in the dataset.
+    
+   - **predict**
+      - The **predict** step uses the ingested dataset for scoring created by the
+        **ingest_scoring** step and applies the specified model to the dataset.
+
+            .. note::
+                In Databricks, the **predict** step writes the output parquet/delta files to
+                DBFS.
 
 .. |'split' step definition in pipeline.yaml| replace:: `'split' step definition in pipeline.yaml <https://github.com/mlflow/mlp-regression-template/blob/35f6f32c7a89dc655fbcfcf731cc1da4685a8ebb/pipeline.yaml#L36-L40>`__
 .. |'register' step definition of pipeline.yaml| replace:: `'register' step definition of pipeline.yaml <https://github.com/mlflow/mlp-regression-template/blob/35f6f32c7a89dc655fbcfcf731cc1da4685a8ebb/pipeline.yaml#L57-L63>`__
 .. |'data' section in pipeline.yaml| replace:: `'data' section in pipeline.yaml <https://github.com/mlflow/mlp-regression-template/blob/35f6f32c7a89dc655fbcfcf731cc1da4685a8ebb/pipeline.yaml#L15-L32>`__
+.. |'data/scoring' section in pipeline.yaml| replace:: `'data/scoring' section in pipeline.yaml <https://github.com/mlflow/mlp-regression-template/blob/f36f3db0f384ab0166789f1978f2b25fa695745c/pipeline.yaml#L34-L38>`__
 .. |'metrics' section of pipeline.yaml| replace:: `'metrics' section of pipeline.yaml <https://github.com/mlflow/mlp-regression-template/blob/35f6f32c7a89dc655fbcfcf731cc1da4685a8ebb/pipeline.yaml#L64-L73>`__
 .. |'validation_criteria' section of the 'evaluate' step definition in pipeline.yaml| replace:: `'validation_criteria' section of the 'evaluate' step definition in pipeline.yaml <https://github.com/mlflow/mlp-regression-template/blob/35f6f32c7a89dc655fbcfcf731cc1da4685a8ebb/pipeline.yaml#L47-L56>`__
+.. |'tuning' section of the 'train' step definition in pipeline.yaml| replace:: `'tuning' section of the 'train' step definition in pipeline.yaml <https://github.com/mlflow/mlp-regression-template/blob/d4ac7ee6ba7649f0d07138565e02402cd7a260c4/pipeline.yaml#L57-L78>`__
 .. |steps/ingest.py| replace:: `steps/ingest.py <https://github.com/mlflow/mlp-regression-template/blob/main/steps/ingest.py>`__
 .. |steps/split.py| replace:: `steps/split.py <https://github.com/mlflow/mlp-regression-template/blob/main/steps/split.py>`__
 .. |steps/train.py| replace:: `steps/train.py <https://github.com/mlflow/mlp-regression-template/blob/main/steps/train.py>`__
@@ -106,34 +129,18 @@ The pipeline steps are defined as follows:
 .. |steps/custom_metrics.py| replace:: `steps/custom_metrics.py <https://github.com/mlflow/mlp-regression-template/blob/main/steps/custom_metrics.py>`__
 """
 
-import os
 import logging
 
-from mlflow.pipelines.regression.v1 import dag_help_strings
-from mlflow.tracking.client import MlflowClient
 from mlflow.pipelines.pipeline import _BasePipeline
 from mlflow.pipelines.steps.ingest import IngestStep, IngestScoringStep
-from mlflow.pipelines.steps.split import (
-    SplitStep,
-    _OUTPUT_TRAIN_FILE_NAME,
-    _OUTPUT_VALIDATION_FILE_NAME,
-    _OUTPUT_TEST_FILE_NAME,
-)
+from mlflow.pipelines.steps.split import SplitStep
 from mlflow.pipelines.steps.transform import TransformStep
 from mlflow.pipelines.steps.train import TrainStep
 from mlflow.pipelines.steps.evaluate import EvaluateStep
-from mlflow.pipelines.steps.predict import (
-    PredictStep,
-    _SCORED_OUTPUT_FILE_NAME,
-)
-from mlflow.pipelines.steps.register import RegisterStep, RegisteredModelVersionInfo
+from mlflow.pipelines.steps.predict import PredictStep
+from mlflow.pipelines.steps.register import RegisterStep
 from mlflow.pipelines.step import BaseStep
-from typing import List, Any, Optional
-from mlflow.pipelines.utils import get_pipeline_root_path
-from mlflow.pipelines.utils.execution import get_or_create_base_execution_directory
-from mlflow.pipelines.utils.execution import get_step_output_path
-from mlflow.exceptions import MlflowException, INVALID_PARAMETER_VALUE
-from mlflow.tracking._tracking_service.utils import _use_tracking_uri
+from typing import Any, Optional
 from mlflow.utils.annotations import experimental
 
 _logger = logging.getLogger(__name__)
@@ -174,8 +181,7 @@ class RegressionPipeline(_BasePipeline):
         regression_pipeline.inspect(step="evaluate")
     """
 
-    _TRAIN_DAG_NAME = "train_dag"
-    _TRAIN_DAG_STEPS = (
+    _PIPELINE_STEPS = (
         # Training data ingestion DAG
         IngestStep,
         # Model training DAG
@@ -184,158 +190,19 @@ class RegressionPipeline(_BasePipeline):
         TrainStep,
         EvaluateStep,
         RegisterStep,
-    )
-
-    _SCORING_DAG_NAME = "scoring_dag"
-    _SCORING_DAG_STEPS = (
         # Batch scoring DAG
         IngestScoringStep,
         PredictStep,
     )
 
-    _PIPELINE_STEPS = _TRAIN_DAG_STEPS + _SCORING_DAG_STEPS
-
-    _STEPS_SUBGRAPH_MAP = dict()
-    for _step_class in _TRAIN_DAG_STEPS:
-        _STEPS_SUBGRAPH_MAP[_step_class] = _TRAIN_DAG_NAME
-    for _step_class in _SCORING_DAG_STEPS:
-        _STEPS_SUBGRAPH_MAP[_step_class] = _SCORING_DAG_NAME
-
-    _SUBGRAPH_INDICES_MAP = {
-        _TRAIN_DAG_NAME: (
-            _PIPELINE_STEPS.index(_TRAIN_DAG_STEPS[0]),
-            _PIPELINE_STEPS.index(_TRAIN_DAG_STEPS[-1]),
-        ),
-        _SCORING_DAG_NAME: (
-            _PIPELINE_STEPS.index(_SCORING_DAG_STEPS[0]),
-            _PIPELINE_STEPS.index(_SCORING_DAG_STEPS[-1]),
-        ),
-    }
-
     _DEFAULT_STEP_INDEX = _PIPELINE_STEPS.index(RegisterStep)
 
     def _get_step_classes(self):
         return self._PIPELINE_STEPS
 
-    def _get_subgraph_for_target_step(self, target_step: BaseStep) -> List[BaseStep]:
-        target_step_class = type(target_step)
-
-        subgraph_name = self._STEPS_SUBGRAPH_MAP[target_step_class]
-        s, e = self._SUBGRAPH_INDICES_MAP[subgraph_name]
-        return self._steps[s : e + 1]
-
     def _get_default_step(self) -> BaseStep:
         return self._steps[self._DEFAULT_STEP_INDEX]
 
-    def _get_pipeline_dag_file(self) -> str:
-        import jinja2
-
-        j2_env = jinja2.Environment(loader=jinja2.FileSystemLoader(os.path.dirname(__file__)))
-        pipeline_dag_template = j2_env.get_template("resources/pipeline_dag_template.html").render(
-            {
-                "pipeline_yaml_help": {
-                    "help_string_type": "yaml",
-                    "help_string": dag_help_strings.PIPELINE_YAML,
-                },
-                "ingest_step_help": {
-                    "help_string": dag_help_strings.INGEST_STEP,
-                    "help_string_type": "text",
-                },
-                "ingest_user_code_help": {
-                    "help_string": dag_help_strings.INGEST_USER_CODE,
-                    "help_string_type": "python",
-                },
-                "ingested_data_help": {
-                    "help_string": dag_help_strings.INGESTED_DATA,
-                    "help_string_type": "text",
-                },
-                "split_step_help": {
-                    "help_string": dag_help_strings.SPLIT_STEP,
-                    "help_string_type": "text",
-                },
-                "split_user_code_help": {
-                    "help_string": dag_help_strings.SPLIT_USER_CODE,
-                    "help_string_type": "python",
-                },
-                "training_data_help": {
-                    "help_string": dag_help_strings.TRAINING_DATA,
-                    "help_string_type": "text",
-                },
-                "validation_data_help": {
-                    "help_string": dag_help_strings.VALIDATION_DATA,
-                    "help_string_type": "text",
-                },
-                "test_data_help": {
-                    "help_string": dag_help_strings.TEST_DATA,
-                    "help_string_type": "text",
-                },
-                "transform_step_help": {
-                    "help_string": dag_help_strings.TRANSFORM_STEP,
-                    "help_string_type": "text",
-                },
-                "transform_user_code_help": {
-                    "help_string": dag_help_strings.TRANSFORM_USER_CODE,
-                    "help_string_type": "python",
-                },
-                "fitted_transformer_help": {
-                    "help_string": dag_help_strings.FITTED_TRANSFORMER,
-                    "help_string_type": "text",
-                },
-                "transformed_training_and_validation_data_help": {
-                    "help_string": dag_help_strings.TRANSFORMED_TRAINING_AND_VALIDATION_DATA,
-                    "help_string_type": "text",
-                },
-                "train_step_help": {
-                    "help_string": dag_help_strings.TRAIN_STEP,
-                    "help_string_type": "text",
-                },
-                "train_user_code_help": {
-                    "help_string": dag_help_strings.TRAIN_USER_CODE,
-                    "help_string_type": "python",
-                },
-                "fitted_model_help": {
-                    "help_string": dag_help_strings.FITTED_MODEL,
-                    "help_string_type": "text",
-                },
-                "mlflow_run_help": {
-                    "help_string": dag_help_strings.MLFLOW_RUN,
-                    "help_string_type": "text",
-                },
-                "custom_metrics_user_code_help": {
-                    "help_string": dag_help_strings.CUSTOM_METRICS_USER_CODE,
-                    "help_string_type": "python",
-                },
-                "evaluate_step_help": {
-                    "help_string": dag_help_strings.EVALUATE_STEP,
-                    "help_string_type": "text",
-                },
-                "model_validation_status_help": {
-                    "help_string": dag_help_strings.MODEL_VALIDATION_STATUS,
-                    "help_string_type": "text",
-                },
-                "register_step_help": {
-                    "help_string": dag_help_strings.REGISTER_STEP,
-                    "help_string_type": "text",
-                },
-                "registered_model_version_help": {
-                    "help_string": dag_help_strings.REGISTERED_MODEL_VERSION,
-                    "help_string_type": "text",
-                },
-                "predict_step_help": {
-                    "help_string": dag_help_strings.PREDICT_STEP,
-                    "help_string_type": "text",
-                },
-            }
-        )
-
-        pipeline_dag_file = os.path.join(
-            get_or_create_base_execution_directory(self._pipeline_root_path), "pipeline_dag.html"
-        )
-        with open(pipeline_dag_file, "w") as f:
-            f.write(pipeline_dag_template)
-
-        return pipeline_dag_file
-
     def run(self, step: str = None) -> None:
         """
         Runs the full pipeline or a particular pipeline step, producing outputs and displaying a
@@ -478,189 +345,7 @@ class RegressionPipeline(_BasePipeline):
             train_df: pd.DataFrame = regression_pipeline.get_artifact("training_data")
             trained_model: PyFuncModel = regression_pipeline.get_artifact("model")
         """
-        import mlflow.pyfunc
-
-        (
-            ingest_step,
-            split_step,
-            transform_step,
-            train_step,
-            _,
-            register_step,
-            ingest_scoring_step,
-            predict_step,
-        ) = self._steps
-
-        def log_artifact_not_found_warning(artifact_name, step_name):
-            _logger.warning(
-                f"The artifact with name '{artifact_name}' was not found."
-                f" Re-run the '{step_name}' step to generate it."
-            )
-
-        def read_run_id():
-            train_output_dir = get_step_output_path(self._pipeline_root_path, train_step.name, "")
-            run_id_file_path = os.path.join(train_output_dir, "run_id")
-            if os.path.exists(run_id_file_path):
-                with open(run_id_file_path, "r") as f:
-                    return f.read().strip()
-            else:
-                return None
-
-        train_step_tracking_uri = train_step.tracking_config.tracking_uri
-        pipeline_root_path = get_pipeline_root_path()
-
-        def read_dataframe_from_path(artifact_path, step_name):
-            import pandas as pd
-
-            if os.path.exists(artifact_path):
-                return pd.read_parquet(artifact_path)
-            else:
-                log_artifact_not_found_warning(artifact_name, step_name)
-                return None
-
-        artifact_path = self._get_artifact_path(
-            artifact_name
-        )  # path may or may not exist, error handling is in this function
-
-        if artifact_name == "ingested_data":
-            return read_dataframe_from_path(artifact_path, ingest_step.name)
-
-        elif artifact_name == "training_data":
-            return read_dataframe_from_path(artifact_path, split_step.name)
-
-        elif artifact_name == "validation_data":
-            return read_dataframe_from_path(artifact_path, split_step.name)
-
-        elif artifact_name == "test_data":
-            return read_dataframe_from_path(artifact_path, split_step.name)
-
-        elif artifact_name == "transformed_training_data":
-            return read_dataframe_from_path(artifact_path, transform_step.name)
-
-        elif artifact_name == "transformed_validation_data":
-            return read_dataframe_from_path(artifact_path, transform_step.name)
-
-        elif artifact_name == "model":
-            run_id = read_run_id()
-            if run_id:
-                with _use_tracking_uri(train_step_tracking_uri, pipeline_root_path):
-                    return mlflow.pyfunc.load_model(f"runs:/{run_id}/{train_step.name}/model")
-            else:
-                log_artifact_not_found_warning("model", train_step.name)
-                return None
-
-        elif artifact_name == "transformer":
-            run_id = read_run_id()
-            if run_id:
-                with _use_tracking_uri(train_step_tracking_uri, pipeline_root_path):
-                    return mlflow.sklearn.load_model(
-                        f"runs:/{run_id}/{transform_step.name}/transformer"
-                    )
-            else:
-                log_artifact_not_found_warning("transformer", train_step.name)
-                return None
-
-        elif artifact_name == "run":
-            run_id = read_run_id()
-            if run_id:
-                with _use_tracking_uri(train_step_tracking_uri, pipeline_root_path):
-                    return MlflowClient().get_run(run_id)
-            else:
-                log_artifact_not_found_warning("mlflow run", train_step.name)
-                return None
-
-        elif artifact_name == "registered_model_version":
-            if os.path.exists(artifact_path):
-                registered_model_info = RegisteredModelVersionInfo.from_json(path=artifact_path)
-                with _use_tracking_uri(train_step_tracking_uri, pipeline_root_path):
-                    return MlflowClient().get_model_version(
-                        name=registered_model_info.name, version=registered_model_info.version
-                    )
-            else:
-                log_artifact_not_found_warning("registered_model_version", register_step.name)
-                return None
-
-        elif artifact_name == "ingested_scoring_data":
-            return read_dataframe_from_path(artifact_path, ingest_scoring_step.name)
-
-        elif artifact_name == "scored_data":
-            return read_dataframe_from_path(artifact_path, predict_step.name)
-
-        else:
-            raise MlflowException(
-                f"The artifact with name '{artifact_name}' is not supported.",
-                error_code=INVALID_PARAMETER_VALUE,
-            )
-
-    def _get_artifact_path(self, artifact_name: str) -> Optional[str]:
-        """
-        Returns a path to an artifact, which may or may not exist depending on whether or not the
-        corresponding pipeline step has been run.
-        """
-        (
-            ingest_step,
-            split_step,
-            transform_step,
-            train_step,
-            _,
-            register_step,
-            ingest_scoring_step,
-            predict_step,
-        ) = self._steps
-
-        if artifact_name == "ingested_data":
-            ingest_output_dir = get_step_output_path(self._pipeline_root_path, ingest_step.name, "")
-            return os.path.join(ingest_output_dir, IngestStep._DATASET_OUTPUT_NAME)
-        elif artifact_name == "training_data":
-            split_output_dir = get_step_output_path(self._pipeline_root_path, split_step.name, "")
-            return os.path.join(split_output_dir, _OUTPUT_TRAIN_FILE_NAME)
-        elif artifact_name == "validation_data":
-            split_output_dir = get_step_output_path(self._pipeline_root_path, split_step.name, "")
-            return os.path.join(split_output_dir, _OUTPUT_VALIDATION_FILE_NAME)
-        elif artifact_name == "test_data":
-            split_output_dir = get_step_output_path(self._pipeline_root_path, split_step.name, "")
-            return os.path.join(split_output_dir, _OUTPUT_TEST_FILE_NAME)
-        elif artifact_name == "transformed_training_data":
-            transform_output_dir = get_step_output_path(
-                self._pipeline_root_path, transform_step.name, ""
-            )
-            return os.path.join(transform_output_dir, "transformed_training_data.parquet")
-        elif artifact_name == "transformed_validation_data":
-            transform_output_dir = get_step_output_path(
-                self._pipeline_root_path, transform_step.name, ""
-            )
-            return os.path.join(transform_output_dir, "transformed_validation_data.parquet")
-        elif artifact_name == "model":
-            train_output_dir = get_step_output_path(self._pipeline_root_path, train_step.name, "")
-            return os.path.join(train_output_dir, "model", "model.pkl")
-        elif artifact_name == "transformer":
-            transform_output_dir = get_step_output_path(
-                self._pipeline_root_path, transform_step.name, ""
-            )
-            return os.path.join(transform_output_dir, "transformer.pkl")
-        elif artifact_name == "run":
-            train_output_dir = get_step_output_path(self._pipeline_root_path, train_step.name, "")
-            return os.path.join(train_output_dir, "run_id")
-        elif artifact_name == "registered_model_version":
-            register_output_dir = get_step_output_path(
-                self._pipeline_root_path, register_step.name, ""
-            )
-            return os.path.join(register_output_dir, "registered_model_version.json")
-        elif artifact_name == "ingested_scoring_data":
-            ingest_scoring_output_dir = get_step_output_path(
-                self._pipeline_root_path, ingest_scoring_step.name, ""
-            )
-            return os.path.join(ingest_scoring_output_dir, IngestScoringStep._DATASET_OUTPUT_NAME)
-        elif artifact_name == "scored_data":
-            predict_output_dir = get_step_output_path(
-                self._pipeline_root_path, predict_step.name, ""
-            )
-            return os.path.join(predict_output_dir, _SCORED_OUTPUT_FILE_NAME)
-        else:
-            raise MlflowException(
-                f"The artifact with name '{artifact_name}' is not supported.",
-                error_code=INVALID_PARAMETER_VALUE,
-            )
+        return super().get_artifact(artifact_name=artifact_name)
 
     def clean(self, step: str = None) -> None:
         """
