@@ -1,7 +1,6 @@
 """Internal utilities for parsing MLproject YAML files."""
 
 import os
-from shlex import quote
 import yaml
 
 from mlflow import data
@@ -9,6 +8,7 @@ from mlflow.exceptions import ExecutionException
 from mlflow.tracking import artifact_utils
 from mlflow.utils.file_utils import get_local_path_or_none
 from mlflow.utils.string_utils import is_string_type
+from mlflow.utils.string_utils import quote
 from mlflow.projects import env_type
 
 MLPROJECT_FILE_NAME = "mlproject"
