@@ -7,7 +7,7 @@ import mlflow
 import mlflow.sagemaker
 from mlflow.sagemaker import DEFAULT_IMAGE_NAME as IMAGE
 from mlflow.utils import cli_args
-from mlflow.utils.annotations import experimental
+from mlflow.utils.annotations import experimental, deprecated
 import mlflow.models.docker_utils
 from mlflow.utils import env_manager as em
 
@@ -111,6 +111,9 @@ def commands():
         " asynchronously using the `--async` flag, this value is ignored."
     ),
 )
+@deprecated(
+    alternative="mlflow deployments create -t sagemaker and mlflow deployments update -t sagemaker"
+)
 def deploy(
     app_name,
     model_uri,
@@ -204,6 +207,7 @@ def deploy(
         " asynchronously using the `--async` flag, this value is ignored."
     ),
 )
+@deprecated(alternative="mlflow deployments delete -t sagemaker")
 def delete(app_name, region_name, archive, asynchronous, timeout):
     """
     Delete the specified application. Unless ``--archive`` is specified, all SageMaker resources
@@ -352,7 +356,7 @@ def delete(app_name, region_name, archive, asynchronous, timeout):
         " asynchronously using the `--async` flag, this value is ignored."
     ),
 )
-@experimental
+@experimental("command")
 def deploy_transform_job(
     job_name,
     model_uri,
@@ -460,7 +464,7 @@ def deploy_transform_job(
         " asynchronously using the `--async` flag, this value is ignored."
     ),
 )
-@experimental
+@experimental("command")
 def terminate_transform_job(job_name, region_name, archive, asynchronous, timeout):
     """
     Terminate the specified Sagemaker batch transform job. Unless ``--archive`` is specified,
@@ -510,7 +514,7 @@ def terminate_transform_job(job_name, region_name, archive, asynchronous, timeou
         )
     ),
 )
-@experimental
+@experimental("command")
 def push_model_to_sagemaker(
     model_name,
     model_uri,
