@@ -2,7 +2,6 @@ import collections
 import inspect
 import logging
 import numpy as np
-import time
 import warnings
 from copy import deepcopy
 from numbers import Number
@@ -12,6 +11,7 @@ from mlflow import MlflowClient
 from mlflow.utils.file_utils import TempDir
 from mlflow.utils.mlflow_tags import MLFLOW_PARENT_RUN_ID
 from mlflow.utils.arguments_utils import _get_arg_names
+from mlflow.utils.time_utils import get_current_time_millis
 
 _logger = logging.getLogger(__name__)
 
@@ -553,6 +553,34 @@ def _log_specialized_estimator_content(
     return metrics
 
 
+def _is_estimator_html_repr_supported():
+    import sklearn
+
+    # Only scikit-learn >= 0.23 supports `estimator_html_repr`
+    return Version(sklearn.__version__) >= Version("0.23.0")
+
+
+def _log_estimator_html(run_id, estimator):
+    if not _is_estimator_html_repr_supported():
+        return
+
+    from sklearn.utils import estimator_html_repr
+
+    # Specifies charset so triangle toggle buttons are not garbled
+    estimator_html_string = f"""
+<!DOCTYPE html>
+<html lang="en">
+  <head>
+    <meta charset="UTF-8"/>
+  </head>
+  <body>
+    {estimator_html_repr(estimator)}
+  </body>
+</html>
+    """
+    MlflowClient().log_text(run_id, estimator_html_string, artifact_file="estimator.html")
+
+
 def _log_estimator_content(
     autologging_client,
     estimator,
@@ -613,7 +641,7 @@ def _log_estimator_content(
             score_key = prefix + "score"
             autologging_client.log_metrics(run_id=run_id, metrics={score_key: score})
             metrics[score_key] = score
-
+    _log_estimator_html(run_id, estimator)
     return metrics
 
 
@@ -723,7 +751,7 @@ def _create_child_runs_for_parameter_search(
     # start time of child runs, since we cannot precisely determine when each point
     # in the parameter search space was explored
     child_run_start_time = parent_run.info.start_time
-    child_run_end_time = int(time.time() * 1000)
+    child_run_end_time = get_current_time_millis()
 
     seed_estimator = cv_estimator.estimator
     # In the unlikely case that a seed of a parameter search estimator is,
