@@ -1,5 +1,3 @@
-import time
-
 import logging
 import sqlalchemy
 
@@ -42,6 +40,7 @@ from mlflow.utils.validation import (
     _validate_model_version,
     _validate_tag_name,
 )
+from mlflow.utils.time_utils import get_current_time_millis
 
 _logger = logging.getLogger(__name__)
 
@@ -54,10 +53,6 @@ _logger = logging.getLogger(__name__)
 sqlalchemy.orm.configure_mappers()
 
 
-def now():
-    return int(time.time() * 1000)
-
-
 class SqlAlchemyStore(AbstractStore):
     """
     This entity may change or be removed in a future release without warning.
@@ -173,7 +168,7 @@ class SqlAlchemyStore(AbstractStore):
             _validate_registered_model_tag(tag.key, tag.value)
         with self.ManagedSessionMaker() as session:
             try:
-                creation_time = now()
+                creation_time = get_current_time_millis()
                 registered_model = SqlRegisteredModel(
                     name=name,
                     creation_time=creation_time,
@@ -234,7 +229,7 @@ class SqlAlchemyStore(AbstractStore):
         """
         with self.ManagedSessionMaker() as session:
             sql_registered_model = self._get_registered_model(session, name)
-            updated_time = now()
+            updated_time = get_current_time_millis()
             sql_registered_model.description = description
             sql_registered_model.last_updated_time = updated_time
             self._save_to_db(session, [sql_registered_model])
@@ -253,7 +248,7 @@ class SqlAlchemyStore(AbstractStore):
         with self.ManagedSessionMaker() as session:
             sql_registered_model = self._get_registered_model(session, name)
             try:
-                updated_time = now()
+                updated_time = get_current_time_millis()
                 sql_registered_model.name = new_name
                 for sql_model_version in sql_registered_model.model_versions:
                     sql_model_version.name = new_name
@@ -377,22 +372,22 @@ class SqlAlchemyStore(AbstractStore):
                         error_code=INVALID_PARAMETER_VALUE,
                     )
                 attr = getattr(SqlRegisteredModel, key)
-                f = SearchUtils.get_sql_filter_ops(attr, comparator, dialect)(value)
-                attribute_filters.append(f)
+                attr_filter = SearchUtils.get_sql_comparison_func(comparator, dialect)(attr, value)
+                attribute_filters.append(attr_filter)
             elif type_ == "tag":
                 if comparator not in ("=", "!=", "LIKE", "ILIKE"):
                     raise MlflowException.invalid_parameter_value(
                         f"Invalid comparator for tag: {comparator}"
                     )
                 if key not in tag_filters:
-                    key_filter = SearchUtils.get_sql_filter_ops(
-                        SqlRegisteredModelTag.key, "=", dialect
-                    )(key)
+                    key_filter = SearchUtils.get_sql_comparison_func("=", dialect)(
+                        SqlRegisteredModelTag.key, key
+                    )
                     tag_filters[key] = [key_filter]
 
-                val_filter = SearchUtils.get_sql_filter_ops(
-                    SqlRegisteredModelTag.value, comparator, dialect
-                )(value)
+                val_filter = SearchUtils.get_sql_comparison_func(comparator, dialect)(
+                    SqlRegisteredModelTag.value, value
+                )
                 tag_filters[key].append(val_filter)
             else:
                 raise MlflowException(
@@ -443,24 +438,26 @@ class SqlAlchemyStore(AbstractStore):
                     # so we already filter out comparison values containing upper case letters
                     # in `SearchModelUtils._get_value`. This addresses MySQL IN clause case
                     # in-sensitive issue.
-                    f = attr.in_(value)
+                    val_filter = attr.in_(value)
                 else:
-                    f = SearchUtils.get_sql_filter_ops(attr, comparator, dialect)(value)
-                attribute_filters.append(f)
+                    val_filter = SearchUtils.get_sql_comparison_func(comparator, dialect)(
+                        attr, value
+                    )
+                attribute_filters.append(val_filter)
             elif type_ == "tag":
                 if comparator not in ("=", "!=", "LIKE", "ILIKE"):
                     raise MlflowException.invalid_parameter_value(
                         f"Invalid comparator for tag: {comparator}",
                     )
                 if key not in tag_filters:
-                    key_filter = SearchUtils.get_sql_filter_ops(
-                        SqlModelVersionTag.key, "=", dialect
-                    )(key)
+                    key_filter = SearchUtils.get_sql_comparison_func("=", dialect)(
+                        SqlModelVersionTag.key, key
+                    )
                     tag_filters[key] = [key_filter]
 
-                val_filter = SearchUtils.get_sql_filter_ops(
-                    SqlModelVersionTag.value, comparator, dialect
-                )(value)
+                val_filter = SearchUtils.get_sql_comparison_func(comparator, dialect)(
+                    SqlModelVersionTag.value, value
+                )
                 tag_filters[key].append(val_filter)
             else:
                 raise MlflowException(
@@ -633,7 +630,7 @@ class SqlAlchemyStore(AbstractStore):
         for tag in tags or []:
             _validate_model_version_tag(tag.key, tag.value)
         with self.ManagedSessionMaker() as session:
-            creation_time = now()
+            creation_time = get_current_time_millis()
             for attempt in range(self.CREATE_MODEL_VERSION_RETRIES):
                 try:
                     sql_registered_model = self._get_registered_model(session, name)
@@ -734,7 +731,7 @@ class SqlAlchemyStore(AbstractStore):
         :return: A single :py:class:`mlflow.entities.model_registry.ModelVersion` object.
         """
         with self.ManagedSessionMaker() as session:
-            updated_time = now()
+            updated_time = get_current_time_millis()
             sql_model_version = self._get_sql_model_version(session, name=name, version=version)
             sql_model_version.description = description
             sql_model_version.last_updated_time = updated_time
@@ -763,7 +760,7 @@ class SqlAlchemyStore(AbstractStore):
             raise MlflowException(msg_tpl.format(stage, DEFAULT_STAGES_FOR_GET_LATEST_VERSIONS))
 
         with self.ManagedSessionMaker() as session:
-            last_updated_time = now()
+            last_updated_time = get_current_time_millis()
 
             model_versions = []
             if archive_existing_versions:
@@ -797,7 +794,7 @@ class SqlAlchemyStore(AbstractStore):
         """
         # currently delete model version still keeps the tags associated with the version
         with self.ManagedSessionMaker() as session:
-            updated_time = now()
+            updated_time = get_current_time_millis()
             sql_model_version = self._get_sql_model_version(session, name, version)
             sql_registered_model = sql_model_version.registered_model
             sql_registered_model.last_updated_time = updated_time
