@@ -11,7 +11,7 @@ test_that("mlflow_start_run()/mlflow_get_run() work properly", {
   run <- mlflow_start_run(
     client = client,
     experiment_id = "0",
-    tags = list(foo = "bar", foz = "baz", mlflow.user = "user1", mlflow.runName = "my_run")
+    tags = list(foo = "bar", foz = "baz", mlflow.user = "user1")
   )
 
   run <- mlflow_get_run(client = client, run$run_uuid)
@@ -24,7 +24,7 @@ test_that("mlflow_start_run()/mlflow_get_run() work properly", {
         list(key = "foz", value = "baz"),
         list(key = "foo", value = "bar"),
         list(key = "mlflow.user", value = "user1"),
-        list(key = "mlflow.runName", value = "my_run")
+        list(key = "mlflow.runName", value = run$run_name)
       )
     )
   )
@@ -59,7 +59,7 @@ test_that("mlflow_end_run() works properly", {
 
   # Verify that only expected run field names are present and that all run info fields are set
   # (not NA).
-  run_info_names <- c("run_uuid", "experiment_id", "user_id", "status", "start_time",
+  run_info_names <- c("run_uuid", "experiment_id", "user_id", "run_name", "status", "start_time",
   "artifact_uri", "lifecycle_stage", "run_id", "end_time")
   run_data_names <- c("metrics", "params", "tags")
   expect_setequal(c(run_info_names, run_data_names), names(run))
@@ -95,7 +95,7 @@ test_that("mlflow_restore_run() work properly", {
   run1 <- mlflow_start_run(
     client = client,
     experiment_id = "0",
-    tags = list(foo = "bar", foz = "baz", mlflow.user = "user1", mlflow.runName = "my_run")
+    tags = list(foo = "bar", foz = "baz", mlflow.user = "user1")
   )
 
   run2 <- mlflow_get_run(client = client, run1$run_uuid)
@@ -111,7 +111,7 @@ test_that("mlflow_restore_run() work properly", {
           list(key = "foz", value = "baz"),
           list(key = "foo", value = "bar"),
           list(key = "mlflow.user", value = "user1"),
-          list(key = "mlflow.runName", value = "my_run")
+          list(key = "mlflow.runName", value = run$run_name)
         )
       )
     )
